import React, { useMemo, useState } from 'react';
import { ControlContainer, TooltipProvider, Tooltip, TooltipTrigger, Button, TooltipContent, Input } from '../../components';
import { Popover, PopoverTrigger, PopoverContent } from '../../components/layout/Popover';
import { useAppDispatch, useGraphQueryResult, useQuerybuilderSettings, useSchemaStats } from '../../data-access';
import { clearQB, QueryBuilderSettings, setQuerybuilderSettings } from '../../data-access/store/querybuilderSlice';
import { QueryMLDialog } from './querysidepanel/QueryMLDialog';
import { QuerySettings } from './querysidepanel/QuerySettings';

export type QueryBuilderToggleSettings = 'settings' | 'ml' | 'logic' | 'relatedNodes' | undefined;

export type QueryBuilderNavProps = {
  toggleSettings: QueryBuilderToggleSettings;
  onFitView: () => void;
  onApplyLayout: () => void;
  onRunQuery: () => void;
  onScreenshot: () => void;
  onLogic: () => void;
  onHide: () => void;
};

export const QueryBuilderNav = (props: QueryBuilderNavProps) => {
  const dispatch = useAppDispatch();
  const qb = useQuerybuilderSettings();
  const result = useGraphQueryResult();
  const resultSize = useMemo(() => (result ? result.edges.length : 0), [result]);

  /**
   * Clears all nodes in the graph.
   */
  function clearAllNodes() {
    dispatch(clearQB());
  }

  return (
    <div className="sticky shrink-0 top-0 flex items-stretch justify-between h-7 bg-secondary-100 border-b border-secondary-200 max-w-full">
      <div className="flex items-center">
        <h1 className="text-xs font-semibold text-secondary-600 px-2 truncate">Query builder</h1>
      </div>
      <div className="sticky right-0 px-0.5 ml-auto items-center flex truncate">
        <ControlContainer>
          <TooltipProvider delayDuration={0}>
            <Tooltip>
              <TooltipTrigger>
                <Button
                  variantType="secondary"
                  variant="ghost"
                  size="xs"
                  iconComponent="icon-[ic--baseline-remove]"
                  onClick={props.onHide}
                />
              </TooltipTrigger>
              <TooltipContent>
                <p>Hide</p>
              </TooltipContent>
            </Tooltip>
            <Tooltip>
              <TooltipTrigger>
                <Button
                  variantType="secondary"
                  variant="ghost"
                  size="xs"
                  iconComponent="icon-[ic--baseline-fullscreen]"
                  onClick={props.onFitView}
                />
              </TooltipTrigger>
              <TooltipContent>
                <p>Fit to screen</p>
              </TooltipContent>
            </Tooltip>
            <Tooltip>
              <TooltipTrigger>
                <Button
                  variantType="secondary"
                  variant="ghost"
                  size="xs"
                  iconComponent="icon-[ic--baseline-delete]"
                  onClick={() => clearAllNodes()}
                />
              </TooltipTrigger>
              <TooltipContent>
                <p>Clear query panel</p>
              </TooltipContent>
            </Tooltip>
            <Tooltip>
              <TooltipTrigger>
                <Button
                  variantType="secondary"
                  variant="ghost"
                  size="xs"
                  iconComponent="icon-[ic--baseline-camera-alt]"
                  onClick={props.onScreenshot}
                />
              </TooltipTrigger>
              <TooltipContent>
                <p>Capture screen</p>
              </TooltipContent>
            </Tooltip>
            <Tooltip>
              <TooltipTrigger>
                <Button
                  variantType="secondary"
                  variant="ghost"
                  size="xs"
                  iconComponent="icon-[ic--baseline-import-export]"
                  onClick={props.onApplyLayout}
                />
              </TooltipTrigger>
              <TooltipContent>
                <p>Layouts</p>
              </TooltipContent>
            </Tooltip>
            <Popover>
              <PopoverTrigger>
                <Tooltip>
                  <TooltipTrigger>
                    <Button
                      variantType="secondary"
                      variant="ghost"
                      size="xs"
                      iconComponent="icon-[ic--baseline-settings]"
                      className="query-settings"
                    />
                  </TooltipTrigger>
                  <TooltipContent>
                    <p>Query builder settings</p>
                  </TooltipContent>
                </Tooltip>
              </PopoverTrigger>
              <PopoverContent>
                <QuerySettings />
              </PopoverContent>
            </Popover>
            <Tooltip>
              <TooltipTrigger>
                <Button
                  variantType="secondary"
                  variant="ghost"
                  size="xs"
                  iconComponent="icon-[ic--baseline-cached]"
                  onClick={props.onRunQuery}
                />
              </TooltipTrigger>
              <TooltipContent>
                <p>Rerun query</p>
              </TooltipContent>
            </Tooltip>
            <Tooltip>
              <TooltipTrigger asChild>
                <Button
                  variantType="secondary"
                  variant="ghost"
                  size="xs"
                  iconComponent="icon-[ic--baseline-difference]"
                  onClick={props.onLogic}
                />
              </TooltipTrigger>
              <TooltipContent disabled={props.toggleSettings === 'logic'}>
                <p>Logic settings</p>
              </TooltipContent>
            </Tooltip>
            <Popover>
              <PopoverTrigger>
                <Tooltip>
                  <TooltipTrigger>
                    <Button variantType="secondary" variant="ghost" size="xs" iconComponent="icon-[ic--baseline-lightbulb]" />
                  </TooltipTrigger>
                  <TooltipContent disabled={props.toggleSettings === 'ml'}>
                    <p>Machine learning</p>
                  </TooltipContent>
                </Tooltip>
              </PopoverTrigger>
              <PopoverContent>
                <QueryMLDialog />
              </PopoverContent>
            </Popover>
            {/* <Tooltip>
            <TooltipTrigger>
              <Button
                variantType="secondary"
                variant="ghost"
                size="xs"
                iconComponent="icon-[ic--baseline-drive-file-move]"
                onClick={() => onAddSchemaToQueryBuilder()}
              />
            </TooltipTrigger>
            <TooltipContent>
              <p>Query All Data</p>
            </TooltipContent>
          </Tooltip> */}
            <Popover>
              <PopoverTrigger>
                <Tooltip>
                  <TooltipTrigger>
                    <Button
                      variantType={qb.limit <= resultSize ? 'danger' : 'secondary'}
                      variant="ghost"
                      size="xs"
                      iconComponent="icon-[ic--baseline-filter-alt]"
                      className={qb.limit <= resultSize ? 'border-danger-600' : ''}
                    />
                  </TooltipTrigger>
                  <TooltipContent disabled={props.toggleSettings === 'ml'}>
                    <p className="font-bold text-base">Limit</p>
                    <p>Limits the number of edges retrieved from the database.</p>
                    <p>Required to manage performance.</p>
                    <p className={`font-semibold${qb.limit <= resultSize ? ' text-danger-800' : ''}`}>
                      Fetched {resultSize} of a maximum of {qb.limit} edges
                    </p>
                  </TooltipContent>
                </Tooltip>
              </PopoverTrigger>
              <PopoverContent>
                <div className="flex flex-col w-full gap-2 px-4 py-2">
                  <span className="text-xs font-bold">Limit</span>
                  <Input
                    type="number"
                    size="xs"
                    value={qb.limit}
                    lazy
                    onChange={(e) => {
                      dispatch(setQuerybuilderSettings({ ...qb, limit: Number(e) }));
                    }}
                    className={`w-24${qb.limit <= resultSize ? ' border-danger-600' : ''}`}
                    containerClassName=""
                  />
                </div>
              </PopoverContent>
            </Popover>
          </TooltipProvider>
        </ControlContainer>
      </div>
    </div>
  );
};
